(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
	typeof define === 'function' && define.amd ? define(factory) :
	(global.Subdivide = factory());
}(this, (function () { 'use strict';

	function noop() {}

	function assign(tar, src) {
		for (var k in src) tar[k] = src[k];
		return tar;
	}

	function appendNode(node, target) {
		target.appendChild(node);
	}

	function insertNode(node, target, anchor) {
		target.insertBefore(node, anchor);
	}

	function detachNode(node) {
		node.parentNode.removeChild(node);
	}

	function createElement(name) {
		return document.createElement(name);
	}

	function createText(data) {
		return document.createTextNode(data);
	}

	function createComment() {
		return document.createComment('');
	}

	function addListener(node, event, handler) {
		node.addEventListener(event, handler, false);
	}

	function removeListener(node, event, handler) {
		node.removeEventListener(event, handler, false);
	}

	function setStyle(node, key, value) {
		node.style.setProperty(key, value);
	}

	function destroyBlock(block, lookup) {
		block.d(1);
		lookup[block.key] = null;
	}

	function updateKeyedEach(old_blocks, component, changed, get_key, dynamic, ctx, list, lookup, node, destroy, create_each_block, intro_method, next, get_context) {
		var o = old_blocks.length;
		var n = list.length;

		var i = o;
		var old_indexes = {};
		while (i--) old_indexes[old_blocks[i].key] = i;

		var new_blocks = [];
		var new_lookup = {};
		var deltas = {};

		var i = n;
		while (i--) {
			var child_ctx = get_context(ctx, list, i);
			var key = get_key(child_ctx);
			var block = lookup[key];

			if (!block) {
				block = create_each_block(component, key, child_ctx);
				block.c();
			} else if (dynamic) {
				block.p(changed, child_ctx);
			}

			new_blocks[i] = new_lookup[key] = block;

			if (key in old_indexes) deltas[key] = Math.abs(i - old_indexes[key]);
		}

		var will_move = {};
		var did_move = {};

		function insert(block) {
			block[intro_method](node, next);
			lookup[block.key] = block;
			next = block.first;
			n--;
		}

		while (o && n) {
			var new_block = new_blocks[n - 1];
			var old_block = old_blocks[o - 1];
			var new_key = new_block.key;
			var old_key = old_block.key;

			if (new_block === old_block) {
				// do nothing
				next = new_block.first;
				o--;
				n--;
			}

			else if (!new_lookup[old_key]) {
				// remove old block
				destroy(old_block, lookup);
				o--;
			}

			else if (!lookup[new_key] || will_move[new_key]) {
				insert(new_block);
			}

			else if (did_move[old_key]) {
				o--;

			} else if (deltas[new_key] > deltas[old_key]) {
				did_move[new_key] = true;
				insert(new_block);

			} else {
				will_move[old_key] = true;
				o--;
			}
		}

		while (o--) {
			var old_block = old_blocks[o];
			if (!new_lookup[old_block.key]) destroy(old_block, lookup);
		}

		while (n) insert(new_blocks[n - 1]);

		return new_blocks;
	}

	function blankObject() {
		return Object.create(null);
	}

	function destroy(detach) {
		this.destroy = noop;
		this.fire('destroy');
		this.set = noop;

		this._fragment.d(detach !== false);
		this._fragment = null;
		this._state = {};
	}

	function _differs(a, b) {
		return a != a ? b == b : a !== b || ((a && typeof a === 'object') || typeof a === 'function');
	}

	function fire(eventName, data) {
		var handlers =
			eventName in this._handlers && this._handlers[eventName].slice();
		if (!handlers) return;

		for (var i = 0; i < handlers.length; i += 1) {
			var handler = handlers[i];

			if (!handler.__calling) {
				handler.__calling = true;
				handler.call(this, data);
				handler.__calling = false;
			}
		}
	}

	function get() {
		return this._state;
	}

	function init(component, options) {
		component._handlers = blankObject();
		component._bind = options._bind;

		component.options = options;
		component.root = options.root || component;
		component.store = component.root.store || options.store;
	}

	function on(eventName, handler) {
		var handlers = this._handlers[eventName] || (this._handlers[eventName] = []);
		handlers.push(handler);

		return {
			cancel: function() {
				var index = handlers.indexOf(handler);
				if (~index) handlers.splice(index, 1);
			}
		};
	}

	function set(newState) {
		this._set(assign({}, newState));
		if (this.root._lock) return;
		this.root._lock = true;
		callAll(this.root._beforecreate);
		callAll(this.root._oncreate);
		callAll(this.root._aftercreate);
		this.root._lock = false;
	}

	function _set(newState) {
		var oldState = this._state,
			changed = {},
			dirty = false;

		for (var key in newState) {
			if (this._differs(newState[key], oldState[key])) changed[key] = dirty = true;
		}
		if (!dirty) return;

		this._state = assign(assign({}, oldState), newState);
		this._recompute(changed, this._state);
		if (this._bind) this._bind(changed, this._state);

		if (this._fragment) {
			this.fire("state", { changed: changed, current: this._state, previous: oldState });
			this._fragment.p(changed, this._state);
			this.fire("update", { changed: changed, current: this._state, previous: oldState });
		}
	}

	function callAll(fns) {
		while (fns && fns.length) fns.shift()();
	}

	function _mount(target, anchor) {
		this._fragment[this._fragment.i ? 'i' : 'm'](target, anchor || null);
	}

	var proto = {
		destroy,
		get,
		fire,
		on,
		set,
		_recompute: noop,
		_set,
		_mount,
		_differs
	};

	const NORTH = 'NORTH';
	const SOUTH = 'SOUTH';
	const EAST = 'EAST';
	const WEST = 'WEST';

	const COLUMN = 'COLUMN';
	const ROW = 'ROW';

	const IS_MAC = navigator.platform === 'MacIntel';
	const KEYCODE = IS_MAC ? 91 : 17; // Cmd : Ctrl

	function clamp(num, min, max) {
		if (num < min) return min;
		if (num > max) return max;
		return num;
	}

	function removeFromArray(array, item) {
		const index = array.indexOf(item);
		if (index === -1) throw new Error('Unexpected error');
		array.splice(index, 1);
	}

	/* src/Pane.html generated by Svelte v2.7.1 */

	const THRESHOLD = 100;

	function wasCmdOrCtrlPressed(event) {
		return IS_MAC ? event.metaKey : event.ctrlKey;
	}

	function cursor({ _keyPressed, edge }) {
		if (!_keyPressed || !edge) return 'default';

		return (
			edge === NORTH ? 's-resize' :
			edge === SOUTH ? 'n-resize' :
			edge === WEST ? 'e-resize' :
			edge === EAST ? 'w-resize' :
			'default'
		);
	}

	function data() {
		return { edge: null };
	}
	var methods = {
		findEdge(event) {
			const { top, right, bottom, left } = this.refs.pane.getBoundingClientRect();

			const d = [
				[NORTH, event.clientY - top],
				[SOUTH, bottom - event.clientY],
				[EAST, right - event.clientX],
				[WEST, event.clientX -left]
			];

			if (d[0][1] > THRESHOLD && d[1][1] > THRESHOLD && d[2][1] > THRESHOLD && d[3][1] > THRESHOLD) return null;

			d.sort((a, b) => a[1] - b[1]);

			return d[0][0];
		},

		handleMousedown(node, event) {
			if (!wasCmdOrCtrlPressed(event)) return;

			const edge = this.findEdge(event);
			if (!edge) return;

			this.fire('split', {
				edge,
				clientX: event.clientX,
				clientY: event.clientY
			});
		},

		handleMousemove(node, event) {
			const edge = this.findEdge(event);
			this.set({ edge });
		}
	};

	function add_css() {
		var style = createElement("style");
		style.id = 'svelte-1afghjx-style';
		style.textContent = ".pane.svelte-1afghjx{position:absolute;overflow:hidden;box-sizing:border-box;padding:calc(var(--thickness) / 2)}.inner.svelte-1afghjx{position:relative;width:100%;height:100%;overflow:hidden}";
		appendNode(style, document.head);
	}

	function create_main_fragment(component, ctx) {
		var div, div_1;

		var switch_value = ctx.component;

		function switch_props(ctx) {
			var switch_instance_initial_data = { pane: ctx.pane };
			return {
				root: component.root,
				data: switch_instance_initial_data
			};
		}

		if (switch_value) {
			var switch_instance = new switch_value(switch_props(ctx));
		}

		function mousedown_handler(event) {
			component.handleMousedown(this, event);
		}

		function mousemove_handler(event) {
			component.handleMousemove(this, event);
		}

		return {
			c() {
				div = createElement("div");
				div_1 = createElement("div");
				if (switch_instance) switch_instance._fragment.c();
				div_1.className = "inner svelte-1afghjx";
				addListener(div, "mousedown", mousedown_handler);
				addListener(div, "mousemove", mousemove_handler);
				div.className = "pane svelte-1afghjx";
				setStyle(div, "left", "" + ctx.pane.getLeft() * 100 + "%");
				setStyle(div, "top", "" + ctx.pane.getTop() * 100 + "%");
				setStyle(div, "width", "" + ctx.pane.getWidth() * 100 + "%");
				setStyle(div, "height", "" + ctx.pane.getHeight() * 100 + "%");
				setStyle(div, "cursor", "" + ctx.cursor + "\n\t");
			},

			m(target, anchor) {
				insertNode(div, target, anchor);
				appendNode(div_1, div);

				if (switch_instance) {
					switch_instance._mount(div_1, null);
				}

				component.refs.pane = div;
			},

			p(changed, ctx) {
				var switch_instance_changes = {};
				if (changed.pane) switch_instance_changes.pane = ctx.pane;

				if (switch_value !== (switch_value = ctx.component)) {
					if (switch_instance) switch_instance.destroy();

					if (switch_value) {
						switch_instance = new switch_value(switch_props(ctx));
						switch_instance._fragment.c();
						switch_instance._mount(div_1, null);
					}
				}

				else if (switch_value) {
					switch_instance._set(switch_instance_changes);
				}

				if (changed.pane) {
					setStyle(div, "left", "" + ctx.pane.getLeft() * 100 + "%");
					setStyle(div, "top", "" + ctx.pane.getTop() * 100 + "%");
					setStyle(div, "width", "" + ctx.pane.getWidth() * 100 + "%");
					setStyle(div, "height", "" + ctx.pane.getHeight() * 100 + "%");
				}

				if (changed.cursor) {
					setStyle(div, "cursor", "" + ctx.cursor + "\n\t");
				}
			},

			d(detach) {
				if (detach) {
					detachNode(div);
				}

				if (switch_instance) switch_instance.destroy();
				removeListener(div, "mousedown", mousedown_handler);
				removeListener(div, "mousemove", mousemove_handler);
				if (component.refs.pane === div) component.refs.pane = null;
			}
		};
	}

	function Pane(options) {
		init(this, options);
		this.refs = {};
		this._state = assign(data(), options.data);
		this._recompute({ _keyPressed: 1, edge: 1 }, this._state);
		this._intro = true;

		if (!document.getElementById("svelte-1afghjx-style")) add_css();

		if (!options.root) {
			this._oncreate = [];
			this._beforecreate = [];
			this._aftercreate = [];
		}

		this._fragment = create_main_fragment(this, this._state);

		if (options.target) {
			this._fragment.c();
			this._mount(options.target, options.anchor);

			this._lock = true;
			callAll(this._beforecreate);
			callAll(this._oncreate);
			callAll(this._aftercreate);
			this._lock = false;
		}
	}

	assign(Pane.prototype, proto);
	assign(Pane.prototype, methods);

	Pane.prototype._recompute = function _recompute(changed, state) {
		if (changed._keyPressed || changed.edge) {
			if (this._differs(state.cursor, (state.cursor = cursor(state)))) changed.cursor = true;
		}
	};

	/* src/Divider.html generated by Svelte v2.7.1 */

	function style({ divider }) {
		const group = divider.parent;

		const x = group.getLeft();
		const y = group.getTop();
		const w = group.getWidth();
		const h = group.getHeight();

		if (divider.type === 'horizontal') {
			return `left: ${x * 100}%; top: ${(y + divider.position * h) * 100}%; width: ${w * 100}%`;
		} else {
			return `top: ${y * 100}%; left: ${(x + divider.position * w) * 100}%; height: ${h * 100}%`;
		}
	}

	function add_css$1() {
		var style = createElement("style");
		style.id = 'svelte-vqvqly-style';
		style.textContent = ".divider.svelte-vqvqly{position:absolute;width:0;height:0;pointer-events:all}.divider.svelte-vqvqly::after{content:'';position:absolute;left:calc(0px - var(--draggable));top:calc(0px - var(--draggable));width:calc(100% + var(--draggable) * 2);height:calc(100% + var(--draggable) * 2)}.horizontal.svelte-vqvqly{cursor:row-resize}.vertical.svelte-vqvqly{cursor:col-resize}.divider.svelte-vqvqly::before{content:'';position:absolute;left:calc(0px - var(--thickness) / 2);top:calc(0px - var(--thickness) / 2);width:calc(100% + var(--thickness));height:calc(100% + var(--thickness));background-color:var(--color)}";
		appendNode(style, document.head);
	}

	function create_main_fragment$1(component, ctx) {
		var div, div_class_value;

		function mousedown_handler(event) {
			component.fire("mousedown", event);
		}

		return {
			c() {
				div = createElement("div");
				addListener(div, "mousedown", mousedown_handler);
				div.className = div_class_value = "" + ctx.divider.type + " divider" + " svelte-vqvqly";
				div.style.cssText = ctx.style;
			},

			m(target, anchor) {
				insertNode(div, target, anchor);
			},

			p(changed, ctx) {
				if ((changed.divider) && div_class_value !== (div_class_value = "" + ctx.divider.type + " divider" + " svelte-vqvqly")) {
					div.className = div_class_value;
				}

				if (changed.style) {
					div.style.cssText = ctx.style;
				}
			},

			d(detach) {
				if (detach) {
					detachNode(div);
				}

				removeListener(div, "mousedown", mousedown_handler);
			}
		};
	}

	function Divider(options) {
		init(this, options);
		this._state = assign({}, options.data);
		this._recompute({ divider: 1 }, this._state);
		this._intro = true;

		if (!document.getElementById("svelte-vqvqly-style")) add_css$1();

		this._fragment = create_main_fragment$1(this, this._state);

		if (options.target) {
			this._fragment.c();
			this._mount(options.target, options.anchor);
		}
	}

	assign(Divider.prototype, proto);

	Divider.prototype._recompute = function _recompute(changed, state) {
		if (changed.divider) {
			if (this._differs(state.style, (state.style = style(state)))) changed.style = true;
		}
	};

	/* src/Subdivide.html generated by Svelte v2.7.1 */

	class Rect {
		constructor(x, y, w, h, prev, next) {
			this.x = x;
			this.y = y;
			this.w = w;
			this.h = h;

			this.prev = prev;
			this.next = next;

			this.parent = null;
		}

		bounds(rect) {
			const width = rect.right - rect.left;
			const height = rect.bottom - rect.top;

			return {
				left: rect.left + width * this.getLeft(),
				top: rect.top + height * this.getTop(),
				width: width * this.getWidth(),
				height: height * this.getHeight()
			};
		}

		getLeft() {
			let left = this.x;

			let node = this;
			while (node = node.parent) left = node.x + (node.w * left);

			return left;
		}

		getTop() {
			let top = this.y;

			let node = this;
			while (node = node.parent) top = node.y + (node.h * top);

			return top;
		}

		getWidth() {
			let width = this.w;

			let node = this;
			while (node = node.parent) width *= node.w;

			return width;
		}

		getHeight() {
			let height = this.h;

			let node = this;
			while (node = node.parent) height *= node.h;

			return height;
		}

		setRange(a, b) {
			if (this.parent.type === COLUMN) {
				this.y = a;
				this.h = (b - a);
			} else {
				this.x = a;
				this.w = (b - a);
			}
		}
	}

	class Pane$1 extends Rect {
		constructor(id, { x, y, w, h, prev, next }) {
			super(x, y, w, h, prev, next);
			this.id = id;
		}

		destroy(panes, dividers) {
			const index = panes.indexOf(this);
			if (index === -1) throw new Error(`Unexpected error`);
			panes.splice(index, 1);
		}
	}

	class Group extends Rect {
		constructor(type, { x, y, w, h, prev, next }) {
			super(x, y, w, h, prev, next);

			this.type = type;
			this.children = [];
			this.dividers = [];
		}

		addChild(child) {
			this.children.push(child);
			child.parent = this;
		}

		replaceChild(child, replacement) {
			const index = this.children.indexOf(child);
			if (index === -1) throw new Error(`Unexpected error`);
			this.children[index] = replacement;

			replacement.parent = this;

			child.parent = replacement;
			replacement.children.push(child);
		}

		destroy(panes, dividers) {
			let i = this.children.length;
			while (i--) this.children[i].destroy(panes, dividers);

			i = this.dividers.length;
			while (i--) this.dividers[i].destroy(panes, dividers);
		}
	}

	class Divider$1 {
		constructor({ id, type, group, position, prev, next }) {
			this.id = id;
			this.type = type;
			this.parent = group;
			this.position = position;
			this.prev = prev;
			this.next = next;

			group.dividers.push(this);
		}

		destroy(panes, dividers) {
			const index = dividers.indexOf(this);
			if (index === -1) throw new Error(`Unexpected error`);
			dividers.splice(index, 1);
		}
	}

	function data$1() {
		const root = new Group(COLUMN, {
			x: 0,
			y: 0,
			w: 1,
			h: 1,
			prev: null,
			next: null
		});

		const pane = new Pane$1(0, {
			x: 0,
			y: 0,
			w: 1,
			h:1,
			prev: null,
			next: null
		});

		root.addChild(pane);

		return {
			thickness: '1px',
			padding: '6px',
			color: 'white',

			_pid: 1,
			_did: 0,
			_panes: [pane],
			_dividers: [],
			_dragging: null,
			_keyPressed: false
		};
	}
	var methods$1 = {
		split(pane, event) {
			const { left, right, top, bottom } = this.refs.container.getBoundingClientRect();
			let { _pid, _did, _panes, _dividers } = this.get();
			const { edge, clientX, clientY } = event;

			const dividerType = edge === NORTH || edge === SOUTH
				? 'horizontal'
				: 'vertical';

			const groupType = dividerType === 'horizontal'
				? COLUMN
				: ROW;

			let group = pane.parent;

			const newGroup = group && group.type === groupType
				? null
				: new Group(groupType, pane);

			if (newGroup) {
				pane.x = pane.y = 0;
				pane.w = pane.h = 1;

				pane.parent.replaceChild(pane, newGroup);
				group = newGroup;

				if (pane.next) pane.next.prev = newGroup;
				if (pane.prev) pane.prev.next = newGroup;

				pane.next = pane.prev = null;
			}

			const bounds = group.bounds(this.refs.container.getBoundingClientRect());

			const newPane = new Pane$1(_pid++, pane);
			group.addChild(newPane);

			const divider = new Divider$1({
				id: _did++,
				type: dividerType,
				group,
				position: null,
				prev: null,
				next: null
			});

			if (groupType === COLUMN) {
				const y = (clientY - bounds.top) / bounds.height;
				const d = y - pane.y;

				divider.position = y;

				if (edge === NORTH) {
					newPane.h = d;
					pane.y = y;
					pane.h -= d;
				} else {
					newPane.y = y;
					newPane.h = pane.h - d;
					pane.h  = d;
				}
			} else {
				const x = (clientX - bounds.left) / bounds.width;
				const d = x - pane.x;

				divider.position = x;

				if (edge === WEST) {
					newPane.w = d;
					pane.x = x;
					pane.w -= d;
				} else {
					newPane.x = x;
					newPane.w = pane.w - d;
					pane.w  = d;
				}
			}

			if (edge === NORTH || edge === WEST) {
				if (pane.prev) pane.prev.next = newPane;

				pane.prev = divider;
				newPane.next = divider;

				divider.prev = newPane;
				divider.next = pane;
			} else {
				if (pane.next) pane.next.prev = newPane;

				pane.next = divider;
				newPane.prev = divider;

				divider.prev = pane;
				divider.next = newPane;
			}

			_panes.push(newPane);
			_dividers.push(divider);

			this.set({
				_pid,
				_did,
				_panes,
				_dividers,
				_dragging: divider
			});

			this._userSelect = document.body.style.userSelect;
			document.body.style.userSelect = 'none';
		},

		start(divider) {
			this._userSelect = document.body.style.userSelect;
			document.body.style.userSelect = 'none';

			this.set({ _dragging: divider });
		},

		drag(event) {
			const { _panes, _dividers, _dragging } = this.get();
			if (!_dragging) return;

			const bounds = _dragging.parent.bounds(this.refs.container.getBoundingClientRect());

			const prev = _dragging.prev;
			const next = _dragging.next;

			let min;
			let max;
			let position;

			if (_dragging.type === 'vertical') {
				min = prev.x;
				max = next.x + next.w;
				position = clamp((event.clientX - bounds.left) / bounds.width, min, max);
			} else {
				min = prev.y;
				max = next.y + next.h;
				position = clamp((event.clientY - bounds.top) / bounds.height, min, max);
			}

			prev.setRange(min, position);
			next.setRange(position, max);

			_dragging.position = position;

			this.set({ _panes, _dividers });
		},

		end(event) {
			const { _panes, _dividers, _dragging } = this.get();
			if (!_dragging) return;

			this.drag(event);

			const prevSize = _dragging.type === 'vertical'
				? _dragging.prev.w
				: _dragging.prev.h;

			const min = Math.min(prevSize, _dragging.type === 'vertical'
				? _dragging.next.w
				: _dragging.next.h);

			if (min < 0.0001) {
				removeFromArray(_dragging.parent.dividers, _dragging);
				removeFromArray(_dividers, _dragging);

				if (prevSize < 0.0001) {
					const mergedDivider = _dragging.prev.prev;

					_dragging.next.prev = mergedDivider;
					if (mergedDivider) mergedDivider.next = _dragging.next;

					_dragging.prev.destroy(_panes, _dividers);
					if (_dragging.prev.parent) {
						removeFromArray(_dragging.prev.parent.children, _dragging.prev);
					}
				} else {
					const mergedDivider = _dragging.next.next;

					_dragging.prev.next = mergedDivider;
					if (mergedDivider) mergedDivider.prev = _dragging.prev;

					_dragging.next.destroy(_panes, _dividers);
					if (_dragging.next.parent) {
						removeFromArray(_dragging.next.parent.children, _dragging.next);
					}
				}
			}

			this.set({ _panes, _dividers, _dragging: false });
			document.body.style.userSelect = this._userSelect;
		},

		handleKeydown(event) {
			this.set({
				_keyPressed: event.which === KEYCODE
			});
		}
	};

	function add_css$2() {
		var style = createElement("style");
		style.id = 'svelte-1xeml4a-style';
		style.textContent = ".clip.svelte-1xeml4a,.overlay.svelte-1xeml4a{top:0;left:0;width:100%;height:100%}.clip.svelte-1xeml4a{position:absolute;overflow:hidden}.overlay.svelte-1xeml4a{position:fixed;z-index:2147483647}.overlay.vertical.svelte-1xeml4a{cursor:col-resize}.overlay.horizontal.svelte-1xeml4a{cursor:row-resize}.layout.svelte-1xeml4a{position:absolute;width:calc(100% + var(--thickness));height:calc(100% + var(--thickness));overflow:hidden;margin:calc(0px - var(--thickness) / 2)}";
		appendNode(style, document.head);
	}

	function create_main_fragment$2(component, ctx) {
		var div, div_1, each_blocks_1 = [], each_lookup = blankObject(), text, each_1_blocks_1 = [], each_1_lookup = blankObject(), text_1;

		function onwindowkeydown(event) {
			component.handleKeydown(event);	}
		window.addEventListener("keydown", onwindowkeydown);

		function onwindowkeyup(event) {
			component.set({ _keyPressed: false });	}
		window.addEventListener("keyup", onwindowkeyup);

		var each_value = ctx._panes;

		const get_key = ctx => ctx.pane.id;

		for (var i = 0; i < each_value.length; i += 1) {
			let child_ctx = get_each_context(ctx, each_value, i);
			let key = get_key(child_ctx);
			each_blocks_1[i] = each_lookup[key] = create_each_block(component, key, child_ctx);
		}

		var each_value_1 = ctx._dividers;

		const get_key_1 = ctx => ctx.divider.id;

		for (var i = 0; i < each_value_1.length; i += 1) {
			let child_ctx = get_each_1_context(ctx, each_value_1, i);
			let key = get_key_1(child_ctx);
			each_1_blocks_1[i] = each_1_lookup[key] = create_each_block_1(component, key, child_ctx);
		}

		var if_block = (ctx._dragging) && create_if_block(component, ctx);

		return {
			c() {
				div = createElement("div");
				div_1 = createElement("div");

				for (i = 0; i < each_blocks_1.length; i += 1) each_blocks_1[i].c();

				text = createText("\n\n\t\t");

				for (i = 0; i < each_1_blocks_1.length; i += 1) each_1_blocks_1[i].c();

				text_1 = createText("\n\n\t\t");
				if (if_block) if_block.c();
				div_1.className = "layout svelte-1xeml4a";
				setStyle(div_1, "--thickness", ctx.thickness);
				setStyle(div_1, "--draggable", "calc(" + ctx.thickness + " + " + ctx.padding + ")");
				setStyle(div_1, "--color", ctx.color);
				div.className = "clip svelte-1xeml4a";
			},

			m(target, anchor) {
				insertNode(div, target, anchor);
				appendNode(div_1, div);

				for (i = 0; i < each_blocks_1.length; i += 1) each_blocks_1[i].m(div_1, null);

				appendNode(text, div_1);

				for (i = 0; i < each_1_blocks_1.length; i += 1) each_1_blocks_1[i].m(div_1, null);

				appendNode(text_1, div_1);
				if (if_block) if_block.m(div_1, null);
				component.refs.container = div_1;
			},

			p(changed, ctx) {
				const each_value = ctx._panes;
				each_blocks_1 = updateKeyedEach(each_blocks_1, component, changed, get_key, 1, ctx, each_value, each_lookup, div_1, destroyBlock, create_each_block, "m", text, get_each_context);

				const each_value_1 = ctx._dividers;
				each_1_blocks_1 = updateKeyedEach(each_1_blocks_1, component, changed, get_key_1, 1, ctx, each_value_1, each_1_lookup, div_1, destroyBlock, create_each_block_1, "m", text_1, get_each_1_context);

				if (ctx._dragging) {
					if (if_block) {
						if_block.p(changed, ctx);
					} else {
						if_block = create_if_block(component, ctx);
						if_block.c();
						if_block.m(div_1, null);
					}
				} else if (if_block) {
					if_block.d(1);
					if_block = null;
				}

				if (changed.thickness) {
					setStyle(div_1, "--thickness", ctx.thickness);
				}

				if (changed.thickness || changed.padding) {
					setStyle(div_1, "--draggable", "calc(" + ctx.thickness + " + " + ctx.padding + ")");
				}

				if (changed.color) {
					setStyle(div_1, "--color", ctx.color);
				}
			},

			d(detach) {
				window.removeEventListener("keydown", onwindowkeydown);

				window.removeEventListener("keyup", onwindowkeyup);

				if (detach) {
					detachNode(div);
				}

				for (i = 0; i < each_blocks_1.length; i += 1) each_blocks_1[i].d();

				for (i = 0; i < each_1_blocks_1.length; i += 1) each_1_blocks_1[i].d();

				if (if_block) if_block.d();
				if (component.refs.container === div_1) component.refs.container = null;
			}
		};
	}

	// (9:2) {#each _panes as pane (pane.id)}
	function create_each_block(component, key_1, ctx) {
		var first;

		var pane_initial_data = {
		 	_keyPressed: ctx._keyPressed,
		 	pane: ctx.pane,
		 	component: ctx.component
		 };
		var pane = new Pane({
			root: component.root,
			data: pane_initial_data
		});

		pane.on("split", function(event) {
			component.split(ctx.pane, event);
		});

		return {
			key: key_1,

			first: null,

			c() {
				first = createComment();
				pane._fragment.c();
				this.first = first;
			},

			m(target, anchor) {
				insertNode(first, target, anchor);
				pane._mount(target, anchor);
			},

			p(changed, _ctx) {
				ctx = _ctx;
				var pane_changes = {};
				if (changed._keyPressed) pane_changes._keyPressed = ctx._keyPressed;
				if (changed._panes) pane_changes.pane = ctx.pane;
				if (changed.component) pane_changes.component = ctx.component;
				pane._set(pane_changes);
			},

			d(detach) {
				if (detach) {
					detachNode(first);
				}

				pane.destroy(detach);
			}
		};
	}

	// (13:2) {#each _dividers as divider (divider.id)}
	function create_each_block_1(component, key_1, ctx) {
		var first;

		var divider_initial_data = { divider: ctx.divider };
		var divider = new Divider({
			root: component.root,
			data: divider_initial_data
		});

		divider.on("mousedown", function(event) {
			component.start(ctx.divider);
		});

		return {
			key: key_1,

			first: null,

			c() {
				first = createComment();
				divider._fragment.c();
				this.first = first;
			},

			m(target, anchor) {
				insertNode(first, target, anchor);
				divider._mount(target, anchor);
			},

			p(changed, _ctx) {
				ctx = _ctx;
				var divider_changes = {};
				if (changed._dividers) divider_changes.divider = ctx.divider;
				divider._set(divider_changes);
			},

			d(detach) {
				if (detach) {
					detachNode(first);
				}

				divider.destroy(detach);
			}
		};
	}

	// (17:2) {#if _dragging}
	function create_if_block(component, ctx) {
		var div, div_class_value;

		function mousemove_handler(event) {
			component.drag(event);
		}

		function mouseup_handler(event) {
			component.end(event);
		}

		function mouseleave_handler(event) {
			component.end(event);
		}

		return {
			c() {
				div = createElement("div");
				addListener(div, "mousemove", mousemove_handler);
				addListener(div, "mouseup", mouseup_handler);
				addListener(div, "mouseleave", mouseleave_handler);
				div.className = div_class_value = "overlay " + ctx._dragging.type + " svelte-1xeml4a";
			},

			m(target, anchor) {
				insertNode(div, target, anchor);
			},

			p(changed, ctx) {
				if ((changed._dragging) && div_class_value !== (div_class_value = "overlay " + ctx._dragging.type + " svelte-1xeml4a")) {
					div.className = div_class_value;
				}
			},

			d(detach) {
				if (detach) {
					detachNode(div);
				}

				removeListener(div, "mousemove", mousemove_handler);
				removeListener(div, "mouseup", mouseup_handler);
				removeListener(div, "mouseleave", mouseleave_handler);
			}
		};
	}

	function get_each_context(ctx, list, i) {
		const child_ctx = Object.create(ctx);
		child_ctx.pane = list[i];
		child_ctx.each_value = list;
		child_ctx.pane_index = i;
		return child_ctx;
	}

	function get_each_1_context(ctx, list, i) {
		const child_ctx = Object.create(ctx);
		child_ctx.divider = list[i];
		child_ctx.each_value_1 = list;
		child_ctx.divider_index = i;
		return child_ctx;
	}

	function Subdivide(options) {
		init(this, options);
		this.refs = {};
		this._state = assign(data$1(), options.data);
		this._intro = true;

		if (!document.getElementById("svelte-1xeml4a-style")) add_css$2();

		if (!options.root) {
			this._oncreate = [];
			this._beforecreate = [];
			this._aftercreate = [];
		}

		this._fragment = create_main_fragment$2(this, this._state);

		if (options.target) {
			this._fragment.c();
			this._mount(options.target, options.anchor);

			this._lock = true;
			callAll(this._beforecreate);
			callAll(this._oncreate);
			callAll(this._aftercreate);
			this._lock = false;
		}
	}

	assign(Subdivide.prototype, proto);
	assign(Subdivide.prototype, methods$1);

	return Subdivide;

})));
